@@ -260,13 +260,15 @@ static std::unique_ptr<TempFile> WriteResponseFile(
 void ProcessArgument(const std::string arg, const std::string developer_dir,
                      const std::string sdk_root, const std::string cwd,
                      bool relative_ast_path, std::string &linked_binary,
-                     std::string &dsym_path, std::string toolchain_path,
+                     std::string &dsym_path, bool &strip_debug_symbols,
+                     std::string toolchain_path,
                      std::function<void(const std::string &)> consumer);
 
 bool ProcessResponseFile(const std::string arg, const std::string developer_dir,
                          const std::string sdk_root, const std::string cwd,
                          bool relative_ast_path, std::string &linked_binary,
-                         std::string &dsym_path, std::string toolchain_path,
+                         std::string &dsym_path, bool &strip_debug_symbols,
+                         std::string toolchain_path,
                          std::function<void(const std::string &)> consumer) {
   auto path = arg.substr(1);
   std::ifstream original_file(path);
@@ -281,7 +283,7 @@ bool ProcessResponseFile(const std::string arg, const std::string developer_dir,
     // unescape them ourselves.
     ProcessArgument(Unescape(arg_from_file), developer_dir, sdk_root, cwd,
                     relative_ast_path, linked_binary, dsym_path,
-                    toolchain_path, consumer);
+                    strip_debug_symbols, toolchain_path, consumer);
   }
 
   return true;
@@ -296,17 +298,18 @@ std::string GetCurrentDirectory() {
 }
 
 std::string exec(std::string cmd) {
-    std::array<char, 128> buffer;
-    std::string result;
-    std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(cmd.c_str(), "r"), pclose);
-    if (!pipe) {
-      std::cerr << "Error: failed to open pipe to '" << cmd << "'" << std::endl;
-      exit(EXIT_FAILURE);
-    }
-    while (fgets(buffer.data(), buffer.size(), pipe.get()) != nullptr) {
-      result += buffer.data();
-    }
-    return result;
+  std::array<char, 128> buffer;
+  std::string result;
+  std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(cmd.c_str(), "r"),
+                                                pclose);
+  if (!pipe) {
+    std::cerr << "Error: failed to open pipe to '" << cmd << "'" << std::endl;
+    exit(EXIT_FAILURE);
+  }
+  while (fgets(buffer.data(), buffer.size(), pipe.get()) != nullptr) {
+    result += buffer.data();
+  }
+  return result;
 }
 
 std::string GetToolchainPath(const std::string &toolchain_id) {
@@ -316,15 +319,16 @@ std::string GetToolchainPath(const std::string &toolchain_id) {
 
   if (output.empty()) {
     std::cerr << "Error: TOOLCHAINS was set to '" << toolchain_id
-      << "' but no toolchain with that ID was found" << std::endl;
+              << "' but no toolchain with that ID was found" << std::endl;
     exit(EXIT_FAILURE);
   } else if (output.find("XcodeDefault.xctoolchain") != std::string::npos) {
     // NOTE: Ideally xcrun would fail if the toolchain we asked for didn't exist
     // but it falls back to the DEVELOPER_DIR instead, so we have to check the
     // output ourselves.
     std::cerr << "Error: TOOLCHAINS was set to '" << toolchain_id
-      << "' but the default toolchain was found, that likely means a matching "
-      << "toolchain isn't installed" << std::endl;
+              << "' but the default toolchain was found, that likely means a "
+                 "matching "
+              << "toolchain isn't installed" << std::endl;
     exit(EXIT_FAILURE);
   }
 
@@ -336,13 +340,14 @@ std::string GetToolchainPath(const std::string &toolchain_id) {
 void ProcessArgument(const std::string arg, const std::string developer_dir,
                      const std::string sdk_root, const std::string cwd,
                      bool relative_ast_path, std::string &linked_binary,
-                     std::string &dsym_path, std::string toolchain_path,
+                     std::string &dsym_path, bool &strip_debug_symbols,
+                     std::string toolchain_path,
                      std::function<void(const std::string &)> consumer) {
   auto new_arg = arg;
   if (arg[0] == '@') {
     if (ProcessResponseFile(arg, developer_dir, sdk_root, cwd,
                             relative_ast_path, linked_binary, dsym_path,
-                            toolchain_path, consumer)) {
+                            strip_debug_symbols, toolchain_path, consumer)) {
       return;
     }
   }
@@ -353,12 +358,17 @@ void ProcessArgument(const std::string arg, const std::string developer_dir,
   if (SetArgIfFlagPresent(arg, "DSYM_HINT_DSYM_PATH", &dsym_path)) {
     return;
   }
+  if (arg == "STRIP_DEBUG_SYMBOLS") {
+    strip_debug_symbols = true;
+    return;
+  }
 
   FindAndReplace("__BAZEL_EXECUTION_ROOT__", cwd, &new_arg);
   FindAndReplace("__BAZEL_XCODE_DEVELOPER_DIR__", developer_dir, &new_arg);
   FindAndReplace("__BAZEL_XCODE_SDKROOT__", sdk_root, &new_arg);
   if (!toolchain_path.empty()) {
-    FindAndReplace("__BAZEL_CUSTOM_XCODE_TOOLCHAIN_PATH__", toolchain_path, &new_arg);
+    FindAndReplace("__BAZEL_CUSTOM_XCODE_TOOLCHAIN_PATH__", toolchain_path,
+                   &new_arg);
   }
 
   // Make the `add_ast_path` options used to embed Swift module references
@@ -419,6 +429,7 @@ int main(int argc, char *argv[]) {
   std::string developer_dir = GetMandatoryEnvVar("DEVELOPER_DIR");
   std::string sdk_root = GetMandatoryEnvVar("SDKROOT");
   std::string linked_binary, dsym_path;
+  bool strip_debug_symbols = false;
 
   const std::string cwd = GetCurrentDirectory();
   std::vector<std::string> invocation_args = {"/usr/bin/xcrun", tool_name};
@@ -432,7 +443,8 @@ int main(int argc, char *argv[]) {
     std::string arg(argv[i]);
 
     ProcessArgument(arg, developer_dir, sdk_root, cwd, relative_ast_path,
-                    linked_binary, dsym_path, toolchain_path, consumer);
+                    linked_binary, dsym_path, strip_debug_symbols,
+                    toolchain_path, consumer);
   }
 
   char *modulemap = getenv("APPLE_SUPPORT_MODULEMAP");
@@ -487,16 +499,32 @@ int main(int argc, char *argv[]) {
     return 0;
   }
 
-  std::vector<std::string> dsymutil_args = {"/usr/bin/xcrun",
-                                            "dsymutil",
-                                            linked_binary,
-                                            "-o",
-                                            dsym_path,
-                                            "--flat",
-                                            "--no-swiftmodule-timestamp"};
+  const std::string bundle_suffix = ".dSYM";
+  bool is_bundle = dsym_path.rfind(bundle_suffix) ==
+                   dsym_path.length() - bundle_suffix.length();
+
+  std::vector<std::string> dsymutil_args = {
+      "/usr/bin/xcrun", "dsymutil",
+      linked_binary,    "-o",
+      dsym_path,        "--no-swiftmodule-timestamp"};
+  if (!is_bundle) {
+    // We should generate a .dSYM bundle only when a path is passed to a .dSYM
+    // directory for backwards compatibility
+    dsymutil_args.push_back("--flat");
+  }
   if (!RunSubProcess(dsymutil_args)) {
     return 1;
   }
 
+  // When stripping is requested, we should still strip the binary
+  // before returning
+  if (strip_debug_symbols) {
+    std::vector<std::string> strip_args = {"/usr/bin/xcrun", "strip", "-S",
+                                           linked_binary};
+    if (!RunSubProcess(strip_args)) {
+      return 2;
+    }
+  }
+
   return 0;
 }
